function [ Equilibrium ] = Func_Solve_The_Model_v0(Parameters,TradeCosts,Options)
% This function takes three structures as inputs and returns a structure with market outcomes as
% output.
% Loading Inputs 
[N, NProv,ProvGroup,SameProv, L, H, beta, sigma, theta, kappa, alpha, phi_min,  EntryCost]=Func_v2struct(Parameters);
[tau_E, tau_P, FixedCost]=Func_v2struct(TradeCosts);
[ToleranceExcessProfit, ToleranceL, ToleranceP,StepsizeWage, StepsizeP, StepsizeM, Max_P_Iter, w_ini, P_power_ini, FirmMass_ini]=Func_v2struct(Options);

% Declare variables here
phi_off=NaN(N,N);   %For the tradeable sector only. Declare the cutoff decision, region by region.
Prob=NaN(N,N);
pi=NaN(N,H);
LaborDemand=NaN(N,H);

%% P Loop Initialization
%Constant Terms for Calculating Profit
DifferenceTerm=((tau_P.^(-theta(2))+tau_E.^(-theta(2))).^(-(1-sigma(2))/theta(2))-tau_E.^(1-sigma(2))).^(1/(1-sigma(2)));
ConstantTerm=(kappa(2)/sigma(2)).^(1/(1-sigma(2)));
Y=w_ini.*L; 

P_gap=1; %Initialization of the P gap.
P_iter_count=0;
Profit_Gap=1; %Initialization of the Profit gap.

%% Main Computation Loop
tic;
while (P_gap>ToleranceP && P_iter_count< Max_P_Iter) || (Profit_Gap>ToleranceExcessProfit)
    P_power_2nd=P_power_ini;
    
    % Wage Loop Initialization
    wage_count=0; 
    ExcessLaborDemand=L;
    w_2nd=w_ini;

    DestinationTerm=(beta(:,2).*Y./w_ini./FixedCost).^(1/(1-sigma(2)))./P_power_ini(:,2).^(1/(1-sigma(2)));
    phi_off_temp=max(ConstantTerm*(w_ini*DestinationTerm').*DifferenceTerm, phi_min(:,2)*ones(1,N));
    phi_off=phi_off_temp-diag(diag(phi_off_temp))+diag(phi_min(:,2));
    Prob=(phi_off./(phi_min(:,2)*ones(1,N))).^(-alpha(2));    
    TempTermA=tau_E.^(1-sigma(2)).*((phi_min(:,2)*ones(1,N)).^alpha(2).*phi_off.^(sigma(2)-alpha(2)-1)-(phi_min(:,2)*ones(1,N)).^(sigma(2)-1)); % Pure E Firm
    TempTermB=(tau_P.^(-theta(2))+tau_E.^(-theta(2))).^(-(1-sigma(2))/theta(2)).*(((phi_min(:,2)*ones(1,N)).^alpha(2)).*phi_off.^(sigma(2)-alpha(2)-1)); % PE Firm
    MatrixO=(TempTermA-TempTermB)*alpha(2)/(sigma(2)-alpha(2)-1);

    LaborTerm1=-(kappa(1)*alpha(1)*(phi_min(:,1).^(ones(N,1)*(sigma(1)-1)))./(sigma(1)-alpha(1)-1)).*beta(:,1)./P_power_ini(:,1)*((sigma(1)-1)/sigma(1)).*FirmMass_ini(:,1);  
    while max(abs(ExcessLaborDemand./L))>ToleranceL &&(wage_count<=100);
        OriginTerm2=w_ini.^(1-sigma(2));
        w_ini=w_2nd;
        Y=w_ini.*L; 
        LaborDemand(:,1)=LaborTerm1.*w_ini.^(-sigma(1)).*Y+EntryCost*FirmMass_ini(:,1);
        DestinationTerm2=beta(:,2).*Y./P_power_ini(:,2);
        LaborDemand(:,2)=((sigma(2)-1)/sigma(2))*kappa(2)*(FirmMass_ini(:,2).*w_ini.^(-sigma(2))).*(MatrixO*DestinationTerm2)+(Prob-diag(ones(N,1)))'*FirmMass_ini(:,2).*FixedCost+EntryCost*FirmMass_ini(:,2);  
        ExcessLaborDemand=sum(LaborDemand,2)-L;
        [~, Index]=max(abs(ExcessLaborDemand./L));
               
        MaxLaborGap=ExcessLaborDemand(Index)/L(Index);
        w_2nd=w_ini+StepsizeWage*w_ini.*max(min((ExcessLaborDemand./L),0.5),-0.5); 
        wage_count=wage_count+1;
    end
    
    PiConstantNT=-(kappa(1)/sigma(1))*(alpha(1)*phi_min(:,1).^(sigma(1)-1)/(sigma(1)-alpha(1)-1)).*w_ini.^(1-sigma(1)).*beta(:,1)./P_power_ini(:,1);
    
    %Block of Code to update Firm Mass   
    TotalFixedCost=(Prob-diag(ones(N,1)))*(FixedCost.*w_ini);
    % Profit Terms
    pi(:,1)=PiConstantNT.*Y; % profit: Sector 1
    pi(:,2)=(kappa(2)/sigma(2))*OriginTerm2.*(MatrixO*DestinationTerm2)-TotalFixedCost;  % profit: Sector 2    
    % Calculate Excess Profit
    ExcessProfit=(pi-EntryCost.*(w_ini*ones(1,2)))./(EntryCost.*(w_ini*ones(1,2)));
    Profit_Gap=max(max(abs(ExcessProfit)));
    % Update Guess of FirmMass
    if Profit_Gap>ToleranceExcessProfit
    FirmMass_ini=max(FirmMass_ini.*(1+StepsizeM*ExcessProfit),0.001);      
    end
    
    %update guess of p_Power.
    P_power_ini(:,1)=-kappa(1)*alpha(1)/(sigma(1)-alpha(1)-1)*(phi_min(:,1).^(sigma(1)-1)).*(FirmMass_ini(:,1).*w_ini.^(1-sigma(1))); % Sector 1
    OriginTerm=FirmMass_ini(:,2).*(w_ini.^(1-sigma(2)));
    P_power_ini(:,2)=kappa(2)*(MatrixO')*OriginTerm; %Sector 2

    P_gap=max(max(abs(log(P_power_2nd)-log(P_power_ini))));
    P_power_ini=(1-StepsizeP)*P_power_2nd+StepsizeP*P_power_ini;
    P_iter_count=P_iter_count+1;
    if mod(P_iter_count,500)==0
        disp([' Wage Iterations =' num2str(wage_count-1) '. Wage Gap=',num2str(MaxLaborGap)])
        disp([' P iterations =',num2str(P_iter_count),' P Gap =',num2str(P_gap),' Profit Gap=',num2str(Profit_Gap)])
    end
end

disp(['Model Solved. P Iterations Completed =',num2str(P_iter_count),'. P Gap =',num2str(P_gap), ' Profit Gap=',num2str(Profit_Gap)])

%% Decomposing Labor Demand
LaborVariableCost=((sigma(2)-1)/sigma(2))*kappa(2)*(FirmMass_ini(:,2).*w_ini.^(-sigma(2))).*(MatrixO*DestinationTerm2);
LaborFixedCost=(Prob-diag(ones(N,1)))'*FirmMass_ini(:,2).*FixedCost;
LaborEntryCost=(EntryCost*FirmMass_ini(:,2));

%% Aggregation
% Inter-City Offline Sales and Online Sales
OriginTerm3=FirmMass_ini(:,1).*w_ini.^(1-sigma(1));
DestinationTerm3=beta(:,1).*Y./P_power_ini(:,1);
IntegralTerm3=alpha(1)*(phi_min(:,1).^(sigma(1)-1))/(sigma(1)-alpha(1)-1);
MatrixTemp= tau_E.^(1-sigma(2)).*(phi_off.^(sigma(2)-alpha(2)-1)- (phi_min(:,2)*ones(1,N)).^(sigma(2)-alpha(2)-1))-(tau_P.^(-theta(2))+tau_E.^(-theta(2))).^(-(1-sigma(2)+theta(2))/theta(2)).* tau_E.^(-theta(2)).*phi_off.^(sigma(2)-alpha(2)-1);
MatrixE=MatrixTemp*alpha(2).*((phi_min(:,2)*ones(1,N)).^alpha(2))/(sigma(2)-alpha(2)-1); 
MatrixP=-((tau_P.^(-theta(2))+tau_E.^(-theta(2))).^(-(1-sigma(2)+theta(2))/theta(2)).* tau_P.^(-theta(2)).*phi_off.^(sigma(2)-alpha(2)-1))*alpha(2).*((phi_min(:,2)*ones(1,N)).^alpha(2))/(sigma(2)-alpha(2)-1);
OriginTerm4=FirmMass_ini(:,2).*w_ini.^(1-sigma(2));
DestinationTerm4=beta(:,2).*Y./P_power_ini(:,2);

Intercity_Sales_Offline=NaN(N,N,H); % Inter-City Offline Sales
Intercity_Sales_Online=NaN(N,N,H);  % Inter-City Online Sales
for h=1:H
     if h==1
     Intercity_Sales_Offline(:,:,h)=diag(-kappa(1)*OriginTerm3.*DestinationTerm3.*IntegralTerm3); 
     Intercity_Sales_Online(:,:,h)=0;            
     else
     Intercity_Sales_Offline(:,:,h)=kappa(h)*(OriginTerm4*DestinationTerm4').*MatrixP; 
     Intercity_Sales_Online(:,:,h)=kappa(h)*(OriginTerm4*DestinationTerm4').*MatrixE;    
     end
end

Intercity_Sales=Intercity_Sales_Online(:,:,2)+Intercity_Sales_Offline(:,:,2);
ImportRatio=(sum(Intercity_Sales,1)'-diag(Intercity_Sales))./sum(Intercity_Sales,1)';
ImportRatioOffline=(sum(Intercity_Sales_Offline(:,:,2),1)'-diag(Intercity_Sales_Offline(:,:,2)))./sum(Intercity_Sales,1)';
ImportRatioOnline=(sum(Intercity_Sales_Online(:,:,2),1)'-diag(Intercity_Sales_Online(:,:,2)))./sum(Intercity_Sales,1)';

Total_Sales_Offline_by_DestinationCity=NaN(N,H);
Total_Sales_Online_by_DestinationCity=NaN(N,H);
% Aggregation of Sales by Destination
for h=1:H
Total_Sales_Offline_by_DestinationCity(:,h)=(ones(1,N)*Intercity_Sales_Offline(:,:,h))';
Total_Sales_Online_by_DestinationCity(:,h)=(ones(1,N)*Intercity_Sales_Online(:,:,h))';
end

InterProvSales_Offline=NaN(NProv,NProv,H); % Inter-Province Offline Sales
InterProvSales_Online=NaN(NProv,NProv,H);  % Inter-Province Online Sale

%Inter-Provincial Sales
for h=1:H
for i=1:NProv
    for j=1:NProv
        InterProvSales_Offline(i,j,h)=(ProvGroup==i)'*(Intercity_Sales_Offline(:,:,h)*(ProvGroup==j));
    end
end
end

for h=1:H
for i=1:NProv
    for j=1:NProv
        InterProvSales_Online(i,j,h)=(ProvGroup==i)'*(Intercity_Sales_Online(:,:,h)*(ProvGroup==j));
    end
end
end

InterProvSales=InterProvSales_Online+InterProvSales_Offline;
ProvAbsorptionRatio=sum(diag(InterProvSales(:,:,2)))/sum(sum(InterProvSales(:,:,2),2),1);
ProvAbsorptionRatio_Online=sum(diag(InterProvSales_Online(:,:,2)))/sum(sum(InterProvSales_Online(:,:,2),2),1);

OnlineShare_by_City=Total_Sales_Online_by_DestinationCity(:,2)./(Total_Sales_Online_by_DestinationCity(:,2)+Total_Sales_Offline_by_DestinationCity(:,2));
OverallOnlineShare=sum(sum(Intercity_Sales_Online(:,:,2),2),1)/sum(sum(Intercity_Sales_Offline(:,:,2)+Intercity_Sales_Online(:,:,2),2),1); % Overall Online Share
InterCityPEFirmPercentage=sum(FirmMass_ini(:,2).*reshape(max(Prob.*(ones(N,N)-eye(N)),[],2),N,1),1)/sum(FirmMass_ini(:,2),1); % Share of Firms that sells through offline channels: 
%%
% Provincial-Level Measure for External Validation Purpose
Total_Sales_Offline_DestinationProvince=sum(InterProvSales_Offline(:,:,2),1)';
Total_Sales_Online_DestinationProvince=sum(InterProvSales_Online(:,:,2),1)';
OfflineAbsorptionRatio_by_Province=diag(InterProvSales_Offline(:,:,2))./Total_Sales_Offline_DestinationProvince;
OnlineAbsorptionRatio_by_Province=diag(InterProvSales_Online(:,:,2))./Total_Sales_Online_DestinationProvince;
OnlineShare_by_Province=Total_Sales_Online_DestinationProvince./(Total_Sales_Offline_DestinationProvince+Total_Sales_Online_DestinationProvince);

%%
% Normalization Step: NominalWage, P_power, PriceIndexNT, PriceIndexT, PriceIndex
NominalWage=w_ini/sum(w_ini.*L, 1);
P_power=zeros(N,H);
P_power(:,1)=P_power_ini(:,1)/(sum(w_ini.*L, 1))^(1-sigma(1)); 
P_power(:,2)=P_power_ini(:,2)/(sum(w_ini.*L, 1))^(1-sigma(2)); 

PriceIndexNT=P_power(:,1).^(1/(1-sigma(1)));
PriceIndexT=P_power(:,2).^(1/(1-sigma(2)));
PriceIndex=(PriceIndexNT.^beta(:,1)).*(PriceIndexT.^beta(:,2));

FirmMass=FirmMass_ini;
OfflineVariety=Prob(:,:)'*FirmMass(:,2);
RealWage=NominalWage./PriceIndex;
AverageFirmSize=sum(L)/sum(sum(FirmMass,2),1);

time=toc/60;
disp(['Percentage selling to any other city in the tradeable sector    = ', num2str(InterCityPEFirmPercentage)])
disp(['Within-Province Sales as a Share of Total Expenditure           = ', num2str(ProvAbsorptionRatio)])
disp(['Average Online Share                                            = ', num2str(OverallOnlineShare)])
disp(['Within-Province Online Sales as a Share of Online Expenditure   = ', num2str(ProvAbsorptionRatio_Online)])
disp(['Time Taken For this Loop                                        = ', num2str(time)])
disp('------------------------------------------------------------------------')

Equilibrium=Func_v2struct(RealWage, NominalWage, PriceIndex, PriceIndexNT,PriceIndexT, FirmMass, OfflineVariety, P_power,...
     Prob,phi_off,InterProvSales_Offline, InterProvSales_Online,InterCityPEFirmPercentage, ProvAbsorptionRatio, ProvAbsorptionRatio_Online, OnlineShare_by_City, OverallOnlineShare,...
     OfflineAbsorptionRatio_by_Province,OnlineAbsorptionRatio_by_Province,OnlineShare_by_Province,...
     ImportRatio, ImportRatioOffline, ImportRatioOnline,AverageFirmSize,...
     LaborVariableCost,LaborFixedCost,LaborEntryCost);
end

