function [ objective_fval ] = Func_SolvingTwoEqm( cost_vec )
%This is the gain-loss function for the estimation of physical trade cost 
% KEY Functions Called: Func_Solve_The_Model_v0,
%                       Func_Solve_The_Model_Mute_v0,
%                       Func_v2struct.
%  Steps: 
%         1. load the data.
%         2. match nominal city by city by updating the
%  city-specific productivity level with a loop
%         3. solve the model twice and calculate the objective function 

load 'BilateralDistance.mat';
load 'SameProv.mat';
load 'data_income.mat';
load 'data_GravityTerms.mat';
load 'CalibrationTargets.mat';
load 'MainParameters'
load 'MainOptions'


[N, NProv, ProvGroup,SameProv, L, H, beta, sigma, theta, kappa, alpha, phi_min, EntryCost]=Func_v2struct(MainParameters);
[DataAverageFirmSize, DataExporterPercentage2002,DataProvAbsorptionRatio2013,DataProvAbsorptionRatioOnline2013,DataOfflineElasticity2002,DataOnlineElasticity2013,DataCoef_L_OnlineShare2013,DataCoef_MP_OnlineShare2013,DataOnlineShare2013]=Func_v2struct(CalibrationTargets);
%% Trade Costs and EComm Costs Matrices

FixedCost=exp(cost_vec(1));
Coef_0_P=cost_vec(2);
Coef_L_P=cost_vec(3);
Coef_Distance_P=cost_vec(4);
Coef_0_E=cost_vec(5);
Coef_DifCity_E=cost_vec(6);
Coef_Distance_E=cost_vec(7);

log_tau_P=Coef_0_P*(ones(N,N)-eye(N))+Coef_L_P*(ones(N,1)*log(L/min(L))')+Coef_Distance_P*log(BilateralDistance);
tau_P=exp(log_tau_P);
tau_P=max(tau_P, 1);

tau_E_2002=exp(50)*ones(N,N); % Setting E-Commerce Cost to be very high.
MyTradeCosts2002=Func_v2struct(tau_E_2002, tau_P, FixedCost);

log_tau_E_2013=Coef_0_E+(ones(N,N)-eye(N))*Coef_DifCity_E+Coef_Distance_E*log(BilateralDistance);
tau_E_2013=exp(log_tau_E_2013);
tau_E_2013=max(tau_E_2013,1);
MyTradeCosts2013=Func_v2struct(tau_E_2013, tau_P, FixedCost);

%% Initial Values for Solving the 2002 Model
try 
    load w_ini2002.mat;
    load P_power_ini2002.mat;  
catch
    disp('Initial Values Not Found, Using Autarky Wage amd Prices ')
    FirmMass=beta.*[L,L]/40;
    w_autarky=ones(N,1);
    P_power_autarky=-((ones(N,1)*(kappa.*alpha)).*phi_min.^(ones(N,1)*(sigma-1))./(ones(N,1)*(sigma-alpha-1))).*(FirmMass.*(w_autarky*ones(1,H)).^(ones(N,1)*(1-sigma))); %From zero profit condition
    w_ini2002=w_autarky;
    P_power_ini2002=P_power_autarky;
end
MainOptions.w_ini=w_ini2002;
MainOptions.P_power_ini=P_power_ini2002; 
%% Matching City-Specific Nominal Income
Step=tic;
data_nominal_income=data_income./data_income(1);
phi_min0=phi_min;
ProductivityVector=ones(N,1);
max_Nominal_Income_gap=1;
NominalIncomeStepSize=0.4;
ToleranceNominalIncome=0.001;
% matching city nominal income
while max_Nominal_Income_gap>ToleranceNominalIncome
    ProductivityVector2=ProductivityVector;
    MainParameters.phi_min=phi_min0.*(ProductivityVector*ones(1,2));
    Eqm=Func_Solve_The_Model_Mute_v0(MainParameters,MyTradeCosts2002,MainOptions);
    model_nominal_income=Eqm.NominalWage./Eqm.NominalWage(1);
    max_Nominal_Income_gap=max(abs((model_nominal_income-data_nominal_income)./data_nominal_income));
    disp(['Max Nominal Income Gap  = ', num2str(max_Nominal_Income_gap)])
    ProductivityVector=ProductivityVector2-NominalIncomeStepSize*ProductivityVector2.*((model_nominal_income-data_nominal_income)./data_nominal_income);
    ProductivityVector=ProductivityVector/ProductivityVector(1);
    MainOptions.w_ini=Eqm.NominalWage;
    MainOptions.P_power_ini=Eqm.P_power;
end
disp('------------------ Nominal Income Matched----------------------------------------------')

%% Saving the productivity parameters
phi_min=MainParameters.phi_min;
currentpath=pwd;
my_phi_min_file=fullfile(currentpath,'Estimates','phi_min.mat');
save(my_phi_min_file,'phi_min')
my_parameter_file=fullfile(currentpath,'Data','MainParameters.mat');
save(my_parameter_file,'MainParameters')
%% Calculating 2002 e-Commerce Moments
Equilibrium2002=Func_Solve_The_Model_v0(MainParameters,MyTradeCosts2002,MainOptions); % Solving the Model without e-Commerce 
InterProvOfflineFlow2002=Equilibrium2002.InterProvSales_Offline(:,:,2);
OfflineGravity2002=reshape(InterProvOfflineFlow2002,NProv*NProv, 1); 
% Gravity of Offline Trade Share 
CoefGravity2002 = regress(log(OfflineGravity2002),data_GravityTerms);
ModelAverageFirmSize=Equilibrium2002.AverageFirmSize;
ModelOfflineElasticity2002=CoefGravity2002(2);
ModelExporterPercentage2002=Equilibrium2002.InterCityPEFirmPercentage;
%% Initial Values for Solving the 2013 Model
try 
    load w_ini2013.mat;
    load P_power_ini2013.mat;  
catch
    w_ini2013=w_ini2002;
    P_power_ini2013=P_power_ini2002;
end
MainOptions.w_ini=w_ini2013;
MainOptions.P_power_ini=P_power_ini2013;
%% Calculating 2013 Moments
Equilibrium2013=Func_Solve_The_Model_v0(MainParameters,MyTradeCosts2013,MainOptions); % Solving the Model with e-Commerce 
ModelOnlineShare2013=Equilibrium2013.OverallOnlineShare;
ModelProvAbsorptionRatio2013=Equilibrium2013.ProvAbsorptionRatio;
ModelProvAbsorptionRatioOnline2013=Equilibrium2013.ProvAbsorptionRatio_Online;
InterProvEcommFlow2013=Equilibrium2013.InterProvSales_Online(:,:,2);
OnlineGravity2013=reshape(InterProvEcommFlow2013,NProv*NProv,1); 
CoefficientGravity2013 = regress(log(OnlineGravity2013),data_GravityTerms);
ModelOnlineElasticity2013=CoefficientGravity2013(2);

%% Regression of Online Expenditure Shares of Log L or Log MP.
MP1=(1./BilateralDistance)*L-(L./diag(BilateralDistance)); %Measure of Market Potential
OnlineShare_by_City2013=Equilibrium2013.OnlineShare_by_City;
coef_all=regress(log(OnlineShare_by_City2013),[ones(N,1),log(L), log(MP1)])'; % Regression of online share on log(L) and log(MP)
Coef_MP_OnlineShare2013=coef_all(end);
Coef_L_OnlineShare2013=coef_all(end-1);

%% Saving Initial Values For Next Iteration
w_ini2002=Equilibrium2002.NominalWage;
P_power_ini2002=Equilibrium2002.P_power;
my_w_file=fullfile(currentpath,'InitialValues','w_ini2002.mat');
save(my_w_file,'w_ini2002')
my_P_power_file=fullfile(currentpath,'InitialValues','P_power_ini2002.mat');
save(my_P_power_file,'P_power_ini2002')

w_ini2013=Equilibrium2013.NominalWage;
P_power_ini2013=Equilibrium2013.P_power;
my_w_file=fullfile(currentpath,'InitialValues','w_ini2013.mat');
save(my_w_file,'w_ini2013')
my_P_power_file=fullfile(currentpath,'InitialValues','P_power_ini2013.mat');
save(my_P_power_file,'P_power_ini2013')

%% Objective Function
objective_fval=((ModelExporterPercentage2002-DataExporterPercentage2002)/(1-DataExporterPercentage2002))^2 ...
    +((Coef_L_OnlineShare2013-DataCoef_L_OnlineShare2013)/DataCoef_L_OnlineShare2013)^2 ...
    +((Coef_MP_OnlineShare2013-DataCoef_MP_OnlineShare2013)/DataCoef_MP_OnlineShare2013)^2 ...
    +((DataProvAbsorptionRatio2013-ModelProvAbsorptionRatio2013)/DataProvAbsorptionRatio2013)^2 ...
    +((DataProvAbsorptionRatioOnline2013-ModelProvAbsorptionRatioOnline2013)/DataProvAbsorptionRatioOnline2013)^2 ...
    +((ModelOnlineShare2013-DataOnlineShare2013)/DataOnlineShare2013)^2 ...
    +((ModelOfflineElasticity2002-DataOfflineElasticity2002)/DataOfflineElasticity2002)^2 ...
    +((ModelOnlineElasticity2013-DataOnlineElasticity2013)/DataOnlineElasticity2013)^2;
%% Display Data Moments
fprintf('------------------------------------------------------------------------------------------- \n');
fprintf('                                      Data        Model \n')
fprintf(' Average Firm Size                  ')
fprintf('%10.4f %10.4f  \n',...
[DataAverageFirmSize,ModelAverageFirmSize])
fprintf(' Share Of Firms Exporting Offline  ')
fprintf('%10.4f %10.4f  \n',...
[DataExporterPercentage2002,ModelExporterPercentage2002])
fprintf(' Province Absorption Ratio         ')
fprintf('%10.4f %10.4f  \n',...
[DataProvAbsorptionRatio2013,ModelProvAbsorptionRatio2013])
fprintf(' Province Absorption Ratio Online  ')
fprintf('%10.4f %10.4f  \n',...
[DataProvAbsorptionRatioOnline2013,ModelProvAbsorptionRatioOnline2013])
fprintf(' Offline Distance Elasticity       ')
fprintf('%10.4f %10.4f  \n',...
[DataOfflineElasticity2002,ModelOfflineElasticity2002])
fprintf(' Online Distance Elasticity        ')
fprintf('%10.4f %10.4f \n',...
[DataOnlineElasticity2013, ModelOnlineElasticity2013])
fprintf(' Online Expenditure Share          ')
fprintf('%10.4f %10.4f \n',...
[DataOnlineShare2013,ModelOnlineShare2013])
fprintf(' Pop Elasticity of Online Share    ')
fprintf('%10.4f %10.4f \n',...
[DataCoef_L_OnlineShare2013,Coef_L_OnlineShare2013])
fprintf(' MP Elasticity of Online Share     ')
fprintf('%10.4f %10.4f \n',...
[DataCoef_MP_OnlineShare2013,Coef_MP_OnlineShare2013])
fprintf(' Welfare Gains from E Commerce         -----    ')
fprintf('%8.4f \n', mean(log(Equilibrium2013.RealWage)-log(Equilibrium2002.RealWage)))

%% Timer
TimeStep=toc(Step)/60;
disp(['Cost Vector Evaluated    = ', num2str(cost_vec)])
disp(['Objective Function       = ', num2str(objective_fval)])
disp(['Minutes taken:', num2str(TimeStep)])
fprintf('----------------------------------------------------------- \n');
end

