/****************************/
* Files needed: ProvValidationData.csv; DataOnlineAbsorption.dta; DataOfflineAbsorption.dta; Data_new_entry_sme.dta; provgroup_concordance.dta
/****************************/
set more off
clear all
set mem 500m

/*Change Directory to the ExternalValidation Subfolder!!!!*/

/*insheet the output file from Program2_validation.m*/
insheet using ProvValidationData.csv, clear
save ProvValidationData.dta, replace

use DataOnlineAbsorption, clear
merge m:1 prov using provgroup_concordance
drop _merge
merge m:1 prov_group using ProvValidationData
drop _merge
merge m:1 prov using DataOfflineAbsorption
drop _merge

rename modelfirmmass2002 ModelFirmMass2002
rename modelfirmmass2013 ModelFirmMass2013
rename modelofflineabsorption ModelOfflineAbsorption 
rename modelonlineabsorption ModelOnlineAbsorption 

gen ln_pop=log(provpop)
gen ln_mp=log(provmp)

order prov_group provname prov provpop ln_pop provmp ln_mp DataOnlineAbsorption DataOfflineAbsorption ModelOnlineAbsorption ModelOfflineAbsorption ModelFirmMass2002 ModelFirmMass2013
gen lnDataOnlineAbsorption=log(DataOnlineAbsorption)
gen lnModelOnlineAbsorption=log(ModelOnlineAbsorption)
gen lnDataOfflineAbsorption=log(DataOfflineAbsorption) 
gen lnModelOfflineAbsorption=log(ModelOfflineAbsorption)

drop if provname=="Xizang"
replace provname="Yunnan" if provname=="Yunan"
merge 1:1 provname using Data_new_entry_sme.dta
drop _merge

*Firm Entry
gen ModelEntry=(ModelFirmMass2013/ModelFirmMass2002-1)
gen DataFirmCountRecent=smeentry2013+smeentry2012+smeentry2011+smeentry2010+smeentry2009+smeentry2008+smeentry2007+smeentry2006
gen DataFirmCountAll=smeentrytotal
gen DataEntry=DataFirmCountRecent/DataFirmCountAll
/* Table 8*/
/*First Row of Table 8*/
corr lnDataOnlineAbsorption  lnDataOfflineAbsorption
corr lnModelOnlineAbsorption lnModelOfflineAbsorption if  lnDataOfflineAbsorption!=.
/*Second Row of Table 8: See Program2_validation.m */
/*Third Row of Table 8*/
corr lnDataOnlineAbsorption  lnModelOnlineAbsorption
/*Fourth Row of Table 8*/
corr DataEntry ModelEntry
/*Figure D1 in Online Appendix*/
label var DataEntry  "Firm entry in the data"
label var ModelEntry "Firm entry in the model"
label var ln_pop "log population"
label var ln_mp "log market potential"
graph twoway scatter DataEntry ModelEntry, mlabel(provname) || lfit DataEntry ModelEntry

# delimit ;
twoway (scatter DataEntry ModelEntry,  mlabel(provname))
	   (lfit DataEntry ModelEntry, lpattern(solid) lwidth(thick) lcolor(maroon))
	   , xtitle("Model") ytitle("Data")
	   legend(off)
	   graphregion(color(white))
;
# delimit cr
graph export figureD1.png, replace


/*Table D5 in Online Appendix*/
reg ModelEntry ln_pop ln_mp, robust
eststo col_1
reg DataEntry ln_pop ln_mp, robust
eststo col_2
reg ModelEntry ln_pop ln_mp if provname!="Chongqing"robust
eststo col_3
reg DataEntry ln_pop ln_mp if provname!="Chongqing", robust
eststo col_4

/*** Display ***/
# delimit ;
esttab col_1 col_2 col_3 col_4, replace
	star(* 0.1 ** 0.05 *** 0.01)
	title(Firm Growth Rate in the Model and the Data)
	mtitle(Model Data Model Data)
	se stats(N r2)
	label
;
# delimit cr
/*END*/
