%% "The Alibaba Effect: Spatial Consumption Inequality and Welfare Gains from E-commerce,"
% Fan, Tang, Zhu and Zou 2018
% This program produces the column for (sigma=5, theta=6) (baseline) of Tables D6, D7 and D8. 

%% House Keeping: managing Directory Paths
restoredefaultpath;
currentpath=pwd;
datapath=fullfile(currentpath,'Data');
estimatepath=fullfile(currentpath,'Estimates');
initialvaluepath=fullfile(currentpath,'InitialValues');
addpath(datapath);
addpath(estimatepath);
addpath(initialvaluepath);
%% Setting Parameter Values
%Load Data
load 'L.mat';
load 'SameProv.mat';
load 'data_service_share.mat';
load 'ProvGroup.mat';
load 'phi_min_v0.mat'
load 'BilateralDistance.mat';
% Key Parameters
N=320    ; %Number of regions
H=2      ; %Number of sectors; h=1 is the nontradeable sector.
sigma=[5,5];
theta=[6,6]; 
beta=[data_service_share,(1-data_service_share)];
alpha=1.3*(sigma-1); % Pareto Shape parameter.
EntryCost=14.286;
kappa=gamma((theta+1-sigma)/theta).*(sigma./(sigma-1)).^(1-sigma); % Kappa is the constant defined in the text. Notice that gamma is a Matlab built-in function  
NProv=max(ProvGroup);    
MainParameters=Func_v2struct(N, NProv, ProvGroup,SameProv, L, H, beta, sigma, theta, kappa, alpha, phi_min, EntryCost);
parameter_file=fullfile(currentpath,'Data','MainParameters.mat');
save(parameter_file,'MainParameters')

load 'L.mat';
load 'SameProv.mat';
load 'data_service_share.mat';
load 'ProvGroup.mat';
load 'phi_min_v0.mat'
load 'BilateralDistance.mat';
load 'MainParameters.mat'
load 'MainOptions.mat'
[N, NProv, ProvGroup,SameProv, L, H, beta, sigma, theta, kappa, alpha, phi_min, EntryCost]=Func_v2struct(MainParameters);
%% Loading Trade Costs and EComm Costs Estimates
load 'calibrated_parameters.mat';
cost_vec=Cost_Vector;
FixedCost=exp(cost_vec(1));
Coef_0_P=cost_vec(2);
Coef_L_P=cost_vec(3);
Coef_Distance_P=cost_vec(4);
Coef_0_E=cost_vec(5);
Coef_DifCity_E=cost_vec(6);
Coef_Distance_E=cost_vec(7);

log_tau_P=Coef_0_P*(ones(N,N)-eye(N))+Coef_L_P*(ones(N,1)*log(L/min(L))')+Coef_Distance_P*log(BilateralDistance);
tau_P=exp(log_tau_P);
tau_P=max(tau_P, 1);

tau_E_2002=exp(50)*ones(N,N); % Setting E-Commerce Cost to be very high.
MyTradeCosts2002=Func_v2struct(tau_E_2002, tau_P, FixedCost);
log_tau_E_2013=Coef_0_E+(ones(N,N)-eye(N))*Coef_DifCity_E+Coef_Distance_E*log(BilateralDistance);
tau_E_2013=exp(log_tau_E_2013);
tau_E_2013=max(tau_E_2013,1);
MyTradeCosts2013=Func_v2struct(tau_E_2013, tau_P, FixedCost);
MarketPotential=(1./BilateralDistance)*L-(L./diag(BilateralDistance)); %Measure of Market Potential
%% Solving the model with the different e-Commerce levels.
disp('-----------------------------' )
disp('  Economy without ECommerce  ' )
NoECommEqm=Func_Solve_The_Model_v0(MainParameters,MyTradeCosts2002,MainOptions);
disp('-----------------------------' )
disp('  Economy with ECommerce     ' )
ECommEqm=Func_Solve_The_Model_v0(MainParameters,MyTradeCosts2013,MainOptions);
%% Tabulating the Effects of e-Commerce
RealWage_by_size_half_M=Func_Summarize_By_Half(NoECommEqm.RealWage,MarketPotential); %summarize
norm_RealWage_M=RealWage_by_size_half_M(1); %getting a number for the normalization. 
RealWage_by_size_half_L=Func_Summarize_By_Half(NoECommEqm.RealWage,L); %summarize
norm_RealWage_L=RealWage_by_size_half_L(1); %getting a number for the normalization. 
% No E
summary_RealWage_NoE_L=Func_Summarize_By_Half(NoECommEqm.RealWage/norm_RealWage_L,L);
summary_RealWage_NoE_M=Func_Summarize_By_Half(NoECommEqm.RealWage/norm_RealWage_M,MarketPotential);
% E
summary_RealWage_E_L=Func_Summarize_By_Half(ECommEqm.RealWage/norm_RealWage_L,L);
summary_RealWage_E_M=Func_Summarize_By_Half(ECommEqm.RealWage/norm_RealWage_M,MarketPotential);
% Changes
% By Population
summary_Change_RealWage_E_L=log(summary_RealWage_E_L)-log(summary_RealWage_NoE_L);
% By Market Potential
summary_Change_RealWage_E_M=log(summary_RealWage_E_M)-log(summary_RealWage_NoE_M);

%% Tables 6,7,8
disp(' Column (sigma=5, theta=6) of Table D6 ')
func= Func_SolvingTwoEqm(Cost_Vector);

% Column 1 of Table D7
delta0=0;
delta1=Coef_0_P+Coef_Distance_P*log(100); % Normalization by which we measure bilateral distance in unit of 100 kms.
delta2=Coef_Distance_P;
deltaL=Coef_L_P;
gamma0=Coef_0_E;
gamma1=Coef_0_E+Coef_DifCity_E+Coef_Distance_E*log(100);  % Normalization by which we measure bilateral distance in unit of 100 kms.
gamma2=Coef_Distance_E;
disp(' Column 1 of Table D7 ')
disp(['     entry cost=     ',num2str(EntryCost(1))])
disp(['     fixed cost=     ',num2str(FixedCost(1))])
disp(['        delta_0=     ',num2str(delta0)])
disp(['        delta_1=     ',num2str(delta1)])
disp(['        delta_2=     ',num2str(delta2)])
disp(['        delta_L=     ',num2str(deltaL)])
disp(['        gamma_0=     ',num2str(gamma0)])
disp(['        gamma_1=     ',num2str(gamma1)])
disp(['        gamma_2=     ',num2str(gamma2)])

% Column 1 of Table D8
fprintf('----------------------------------------------------------- \n');
fprintf(' Column 1 of Table D8                   \n');
fprintf(' The Effect of E-Commerce on  Real Wage \n');
fprintf('      Change                         Quintile of City Population      Quintile of City Market Potential     \n')
fprintf('                   Variable          Above Median    Below Median      Above Median    Below Median          \n')
fprintf('                     Real Wage')
fprintf('%15.4f %15.4f %15.4f %15.4f \n',...
[summary_Change_RealWage_E_L,summary_Change_RealWage_E_M])
% END
